#!/usr/bin/perl

package Document;

##############################################################
## This class represents the xml file in memory. This has	##
## functions to load the input xml, update the module nodes	##
## update the user profile and finally write the xml back	##
## to the given output file.								##
##############################################################


use XML::LibXML;
use global;
use strict;
use Advisor;
use Analyzer;

## This function is a constructor that loads the given
 # file onto a root node element that can be traversed
 # to read and load the required vlaue.
 ## 
sub new {
	if ($global::DEBUG) {
		print "Document.Constructor() : Initializing the new Dcoument element\n";
	}
	my $class = shift;
	my $self = {
		fileName => shift,
		rootDoc => undef,
		format => 0, ## this is to use the input format as it is in output
		rootNodeName => "/Java_Perf_Advisor/",
		valueNodeName => "PerfValue",
		metricNodeName => "PerformanceCategory",
		profileNodeName => "Performance_Profile",
		jobNodeName => "Job_Information",
		machineNodeName => "Trace_Information",
	};
	bless $self, $class;
	$self->_parseFile();
    return $self;
}


## This function can be used to change the filename
 # of the previously loaded file and reload a new file
 ##
sub loadFile {
	my ( $self, $file ) = @_;
    $self->{fileName} = $file;
    if ($global::DEBUG) {
		print "Document.loadFile() : Loading the new file '".($self->{fileName})."'\n";
	}
    $self->_parseFile();
}


## As the name indicate this function is used to
 # print the doc in the xml format. If the file 
 # name is given as an argument then the doc is
 # saved to file else it is printed on the screen
 ##
sub printDoc {
	my ($self, $file) = @_;
	if ($file eq "") {
		## can be used for debug only
		my @doc = $self->{rootDoc}->findnodes($self->{rootNodeName}.".");
		print $doc[0]->toString($self->{format});
	} else {
		$self->{rootDoc}->toFile($file, $self->{format});
	}
}


## This is a class private function that is used
 # parse the given xml document and load into memory
 ##
sub _parseFile {
	my $self = shift;
	if ($global::DEBUG) {
		print "Document._parseFile() : Parsing the file '".($self->{fileName})."'\n";
	}
	my $parser = XML::LibXML->new();
	$self->{rootDoc} = $parser->parse_file($self->{fileName});
	# check the version of the xml file
	if ($global::DEBUG) {
		print "Document._parseFile() : Checking the input xml file version.\n";
	}
    my @rootNode = $self->{rootDoc}->findnodes($self->{rootNodeName}.".");
    if ($rootNode[0]->getAttribute('version') ne $global::VERSION) {
    	print STDERR "Error: The input xml file is not in the correct version\n";
		exit;
    }
    if ($global::DEBUG) {
		print "Document._parseFile() : Version : ".$global::VERSION."\n";
	}
}


## This function is used to modify the 
 # the profile node of xml document
 ##
sub updateProfileNode {
	my $self = shift;
	my ($adminExperiance, $machineType, $workloadType) = @_;
	my @profileNodes = $self->{rootDoc}->findnodes($self->{rootNodeName}.$self->{profileNodeName});
	foreach my $inputNode ($profileNodes[0]->getChildrenByTagName("Profile_Input")) {
		my @adminInputNode = $inputNode->getChildrenByTagName('Admin_Input');
		if ($inputNode->getAttribute('Name') eq "Admin Experience") {
			$adminInputNode[0]->appendText($adminExperiance);
		} elsif ($inputNode->getAttribute('Name') eq "Java Importance on Partition") {
			$adminInputNode[0]->appendText($workloadType);
		} elsif ($inputNode->getAttribute('Name') eq "System Usage") {
			$adminInputNode[0]->appendText($machineType);
		}
	}
	if ($global::DEBUG) {
		print "Document.updateProfileNode() : Updated the profile node\n";
	}
	## even update the job related info here
	my @jobNode = $self->{rootDoc}->findnodes($self->{rootNodeName}.$self->{jobNodeName});
	my @node = $jobNode[0]->getChildrenByTagName("PID");
	$node[0]->appendText($global::Process{"Id"});
	my @node = $jobNode[0]->getChildrenByTagName("User");
	$node[0]->appendText($global::Process{"User"});
	my @node = $jobNode[0]->getChildrenByTagName("CommandLine");
	$node[0]->appendText($global::Process{"CommandLine"});
	if ($global::DEBUG) {
		print "Document.updateProfileNode() : Updated the profile node\n";
	}
	## also update the date, time and hostname of the machine
	my @machineNode = $self->{rootDoc}->findnodes($self->{rootNodeName}.$self->{machineNodeName});
	my @node = $machineNode[0]->getChildrenByTagName("HostName");
	$global::Hostname = `hostname`;
	chop $global::Hostname;
	$node[0]->appendText($global::Hostname);
	my @node = $machineNode[0]->getChildrenByTagName("TimeTaken");
	my $time = `date +"%H:%M"`;
	chop $time;
	$node[0]->appendText($time);
	my @node = $machineNode[0]->getChildrenByTagName("DateTaken");
	my $date = `date +"%b %d, %Y"`;
	chop $date;
	$node[0]->appendText($date);
	if ($global::DEBUG) {
		print "Document.updateProfileNode() : Updated the host information\n";
	}
}


## This function is used to modify the xml document
 # @category - perf category name like java, WAS...
 # @associative array of values
 ##
sub updateModuleNode {
	my ($self, $categoryName) = @_;
	my $currentValues = $global::Modules{$categoryName};
	my $categoryNode = undef;
	# loop through the 4 category nodes and get the required one
	foreach my $child ($self->{rootDoc}->findnodes($self->{rootNodeName}.$self->{metricNodeName})) {
		if ($child->getAttribute('Name') eq $categoryName) {
			$categoryNode = $child;
		}
	}
	# if the given category is not found
	if (!$categoryNode) {
		print STDERR "Error: Could not find the specified node '$categoryName'!\n";
		exit;
	}
	if ($global::DEBUG) {
		print "Document.updateModuleNode() : Updating the $categoryName node\n";
	}
	if ($currentValues->{"IsValid"} == 0) {
		# if this node is not required then delete it from the doc
		my $parentNode = $categoryNode->parentNode();
		$parentNode->removeChild($categoryNode);
		if ($global::DEBUG) {
			print "Document.updateModuleNode() : $categoryName node is not valid. Removing the node\n";
		}
		return;
	}
	# if the category is found get all the perfValue node of it and update the current value
	foreach my $perfValueNode ($categoryNode->getChildrenByTagName($self->{valueNodeName})) {
		## update the current value here
		my $nodeName = $perfValueNode->getAttribute('Name');
		my $nodeValue = $currentValues->{$nodeName};
		# icon to set in the ui
		my $nodeIcon = "None";
		if ($nodeValue eq "") {
			#if the value is not set then remove the node
			$categoryNode->removeChild($perfValueNode);
			if ($global::DEBUG) {
				print "Document.updateModuleNode() : No current value set for $nodeName. Removing this metric\n";
			}
		} else {
			# if there is a value then update the node
			my @currentValueNode = $perfValueNode->getChildrenByTagName('CurrentValue');
			$currentValueNode[0]->appendText($nodeValue);
			
			# check if its a Valid node to get recommendation
			my $isRec = 1;
			if ($perfValueNode->getAttribute('ReadOnly') eq "Y") {
				$isRec = 0;
				if ($global::DEBUG) {
					print "Document.updateModuleNode() : \"$nodeName\" node is ReadOnly. No Recommendation required.\n";
				}
			} elsif ($perfValueNode->getAttribute('Hidden') eq "Y") {
				$isRec = 0;
				#set it to "U" to unhide if -internal is specified
				if ($global::SHOW_HIDDEN) {
					($perfValueNode->getAttributeNode('Hidden'))->setValue("U");
				}
				if ($global::DEBUG) {
					print "Document.updateModuleNode() : \"$nodeName\" node is Hidden. No Recommendation required.\n";
				}
			}
			
			## check if Risk needs to be calculated
			if (($perfValueNode->getChildrenByTagName('Risk'))[0]->textContent eq "") {
				my $risk = Analyzer::getRisk($categoryName, $nodeName);
				($perfValueNode->getChildrenByTagName('Risk'))[0]->appendText($risk);
			} #end of if Risk

			## check if Impact needs to be calculated
			if (($perfValueNode->getChildrenByTagName('Impact'))[0]->textContent eq "") {
				my $impact = Analyzer::getImpact($categoryName, $nodeName);
				($perfValueNode->getChildrenByTagName('Impact'))[0]->appendText($impact);
			} #end of if Impact
			
			if ($isRec) {
				## if the value is not read only, then get the recommendation
				my $giveAdvice = 0;
				## find the risk and impact and calculate the score for this metric
				my $risk = ($perfValueNode->getChildrenByTagName('Risk'))[0]->textContent;
				my $impact = ($perfValueNode->getChildrenByTagName('Impact'))[0]->textContent;
				my $nodeScore = $global::Score + $impact - $risk;			
				# if there is a minRecomend node
				if ($perfValueNode->getChildrenByTagName('MinRecommendedValue')) {
					my $maxValue = ($perfValueNode->getChildrenByTagName('MaxRecommendedValue'))[0]->textContent;
					my $minValue = ($perfValueNode->getChildrenByTagName('MinRecommendedValue'))[0]->textContent;
					#if values are empty then get it from the advisor using the algorithm
					if ($minValue eq "") {
						($minValue, $maxValue) = Advisor::getRecommendation($categoryName, $nodeName);
						($perfValueNode->getChildrenByTagName('MinRecommendedValue'))[0]->appendText($minValue);
						($perfValueNode->getChildrenByTagName('MaxRecommendedValue'))[0]->appendText($maxValue);
					}
					if ($global::DEBUG) {
						print "Document.updateModuleNode() : Module=$categoryName Metric=$nodeName  CurrentValue=$nodeValue Recommendation=$minValue to $maxValue Score=$nodeScore ";
					}
					# if node value is a string like "unlimited" replace it as infinity
					if (uc($nodeValue) eq "UNLIMITED") {
						$nodeValue = $global::INFINITE;
					}
					if (uc($maxValue) eq "UNLIMITED") {
            					$maxValue = $global::INFINITE+1;
 					}
	
	
					#if minRecomend is greater than the currentValue then set the advice
					if ($minValue > $nodeValue) {
						
						$giveAdvice = 1;
					} elsif (($maxValue ne "NoMax") && ($maxValue < $nodeValue)) {
						
						$giveAdvice = 1;
					}
				} else {
					my $recValue = ($perfValueNode->getChildrenByTagName('RecommendedValue'))[0]->textContent;
					#if values are empty then get it from the advisor using the algorithm
					if ($recValue eq "") {
						$recValue = Advisor::getRecommendation($categoryName, $nodeName);
						($perfValueNode->getChildrenByTagName('RecommendedValue'))[0]->appendText($recValue);
					}
					if ($global::DEBUG) {
						print "Document.updateModuleNode() : Module=$categoryName Metric=$nodeName CurrentValue=$nodeValue Recommendation=$recValue Score=$nodeScore ";
					}
					if ($recValue eq "") {
						$giveAdvice = 0;
					} elsif ($recValue eq "More Details...") {
						$giveAdvice = -1;	#more details icon
					} elsif ($recValue =~ /^-?[\.|\d]*\Z/) {
						# if node value is a string like "unlimited" replace it as infinity
						if (uc($nodeValue) eq "UNLIMITED") {
							$nodeValue = $global::INFINITE;
						}
						#check for numeric values
						if ($recValue != $nodeValue) {
							$giveAdvice = 1;
						}
					} elsif (uc($recValue) ne uc($nodeValue)) {
						$giveAdvice = 1;
					}
				} #end of if minRec or RecVal
				# decide on the mode of recomendation
				$nodeIcon = "Good";
				if ($giveAdvice == 1) {
					if ($nodeScore > $global::MIN_VAL_REC) {
						$nodeIcon = "Recommendation";
					} else {
						$nodeIcon = "Warning";
					}
				} elsif ($giveAdvice == -1) {
					$nodeIcon = "Investigate";
				}
				if ($global::DEBUG) {
					print " giveAdvice=$giveAdvice NodeIcon=$nodeIcon\n";
				}
			} # end of if !ReadOnly
			## set the recommended change here
			($perfValueNode->getChildrenByTagName('RecommendedChange'))[0]->appendText($nodeIcon);
		} # end of if not delete node
	}
}


1;
